#!/usr/bin/env python
# PYTHON_ARGCOMPLETE_OK
import argparse
import sys, os
from decocare import link, stick, session, commands, lib
from pprint import pformat
import logging
import time
log = logging.getLogger( ).getChild(__name__)

def parse_env ( ):
  return {
   "serial": os.environ.get('SERIAL', ''),
   "port": os.environ.get('PORT', '')
  }


class CommandApp(object):
  def __init__(self):
    self.env = parse_env( )
    self.parser = self.get_parser( )
    self.autocomplete( )

  def autocomplete (self):
    try:
      import argcomplete
      argcomplete.autocomplete(self.parser)
    except ImportError:
      # no auto completion
      pass

  def help (self):
    return self.__class__.__doc__

  def get_parser (self):
    conf = parse_env( )
    helptext = self.help( ).split("\n")
    description = '\n'.join(helptext[0:2])
    epilog = '\n'.join(helptext[2:])
    parser = argparse.ArgumentParser(description=description, epilog=epilog)
    parser.add_argument('--serial', type=str,
                        dest='serial',
                        default=conf.get('serial', ''),
                        help="serial number of pump [default: %(default)s]")
    parser.add_argument('--port', type=str,
                        dest='port',
                        default=conf.get('port', ''),
                        help="Path to device [default: %(default)s]")
    parser.add_argument('--no-op',
                        dest='dryrun',
                        action='store_true', default=False,
                        help="Dry run, don't do main function"
                        )
    parser.add_argument('--skip-prelude',
                        dest='no_prelude',
                        action='store_true', default=False,
                        help="Don't do the normal prelude."
                        )
    parser.add_argument('--no-rf-prelude',
                        dest='no_rf_prelude',
                        action='store_true', default=False,
                        help="Do the prelude, but don't query the pump."
                        )
    parser.add_argument('--skip-postlude',
                        dest='no_postlude',
                        action='store_true', default=False,
                        help="Don't do the normal postlude."
                        )
    parser.add_argument('-v', '--verbose',
                        dest='verbose',
                        action='append_const', const=1,
                        help="Verbosity"
                        )
    parser.add_argument('--init',
                        dest='init',
                        action='store_true', default=False,
                        help="Send power ctrl to initialize RF session."
                        )
    parser = self.customize_parser(parser)
    return parser

  def customize_parser (self, parser):
    parser.add_argument('commands',
                        nargs="+",
                        #dest='status',
                        choices=['act', 'esc', 'up', 'down', 'easy' ],
                        default='act',
                        help="buttons to press [default: %(default)s)]"
                        )
    return parser

  def configure (self):
    args = self.parser.parse_args( )
    level = None
    if args.verbose > 0:
      level = args.verbose > 1 and logging.DEBUG or logging.INFO
    logging.basicConfig(stream=sys.stdout, level=level)
    self.args = args
    return args

  def run (self, args):
    if not args:
      args = self.configure( )
    self.prelude(args)
    self.main(args)
    self.postlude(args)

  def prelude (self, args):
    if args.no_prelude:
      print "##### skipping prelude"
      return
    print "## do stuff with an insulin pump over RF"
    print "using", "`", args, "`"

    print "```"
    uart = stick.Stick(link.Link(args.port, timeout=.400))
    print "```"
    print "```"
    uart.open( )
    print "```"
    print "```"
    pump = session.Pump(uart, args.serial)
    print "```"
    print "```"
    stats = uart.interface_stats( )
    print "```"
    print "```javascript"
    print pformat(stats)
    print "```"
    self.uart = uart
    self.pump = pump
    self.model = None
    if args.no_rf_prelude:
      print "##### skipping normal RF preludes"
      return
    print "```"
    if args.init:
      pump.power_control( )
    model = pump.read_model( )
    self.model = model
    print "```"
    print '### PUMP MODEL: `%s`' % model

  def postlude (self, args):
    if args.no_postlude:
      print "##### skipping postlude"
      return
    print "### end stats"
    print "```"
    stats = self.uart.interface_stats( )
    print "```"
    print "```javascript"
    print pformat(stats)
    print "```"

  def main (self, args):
    for flow in args.commands:
      print '### ', flow
      if args.dryrun:
        print "#### dry run, no action taken"
      else:
        exec_request(self.pump, flow)

command_map = {
    'easy': commands.KeypadPush.EASY,
    'esc': commands.KeypadPush.ESC,
    'act': commands.KeypadPush.ACT,
    'down': commands.KeypadPush.DOWN,
    'up': commands.KeypadPush.UP,
}

def exec_request (pump, msg, args={},
                  dryrun=False,
                  save=False,
                  prefix='',
                  render_decoded=True,
                  render_hexdump=True):
  if dryrun:
    print "skipping query", pump, msg, args
    return False
  response = pump.query(msg, **args)
  print "response: %s" % response
  if render_hexdump:
    print "hexdump:"
    print "```python"
    print response.hexdump( )
    print "```"
  if render_decoded:
    print "#### decoded:"
    print "```python"
    print repr(response.getData( ))
    print "```"
  if save:
    response.save(prefix=prefix)
  return response

class SendMsgApp(CommandApp):
  """
  %(prog)s - send messages to a compatible MM insulin pump

  This tool is intended to help discover protocol behavior.
  Under no circumstance is it intended to deliver therapy.
  """
  def main (self, args):
    if args.prefix:
      self.execute_list(args.prefix, args.saveall)
    if args.command == "ManualCommand":
      kwds = {
          'params': map(int, getattr(args, 'params', [ ])),
          'retries': getattr(args, 'retries', 0),
          'effectTime': getattr(args, 'effectTime'),
          'maxRecords': getattr(args, 'maxRecords'),
          'bytesPerRecord': getattr(args, 'bytesPerRecord'),
          'code': args.code,
          'name': getattr(args, 'name', "ExperimentCode%02x" % args.code),
          'descr': getattr(args, 'descr', "Experimental msg")
        }
      msg = commands.ManualCommand
      resp = exec_request(self.pump, msg, args=kwds,
                   dryrun=args.dryrun, render_hexdump=False, save=args.save, prefix=args.prefix_path)
    if args.command == "sleep":
      time.sleep(args.timeout)
    if args.command == "tweak":
      Other = getattr(commands, args.other)
      kwds = commands.TweakAnotherCommand.get_kwds(Other, args)
      print Other, kwds
      resp = exec_request(self.pump, Other, args=kwds,
                   dryrun=args.dryrun, save=args.save, prefix=args.prefix_path)
    if args.postfix:
      self.execute_list(args.postfix, args.saveall)

  def execute_list (self, messages, save=False):
    for name in messages:
      msg = getattr(commands, name)
      print "###### sending `%s`" % getattr(msg, 'name', msg)
      resp = exec_request(self.pump, msg, dryrun=self.args.dryrun,
                          render_hexdump=self.args.verbose>0,
                          save=save, prefix=self.args.prefix_path)

  def customize_parser (self, parser):
    choices = commands.__all__
    parser.add_argument('--prefix-path',
                        dest="prefix_path",
                        type=str,
                        default="",
                        help="Prefix to store saved files when using --save or --saveall."
                        )
    parser.add_argument('--saveall',
                        action="store_true",
                        default=False,
                        help="Whether or not to save all responses.",
                        )

    parser.add_argument('--prefix',
                        action="append",
                        choices=choices,
                        help="Built-in commands to run before the main one."
                        )

    parser.add_argument('--postfix',
                        action="append",
                        choices=choices,
                        help="Built-in commands to run after the main one."
                        )

    subparsers = parser.add_subparsers(help="Main thing to do between --prefix and--postfix", dest="command")
    sleep_parser = subparsers.add_parser("sleep", help="Just sleep between command sets")
    sleep_parser.add_argument('timeout', type=float, default=0,
                              help="Sleep in between running --prefix and --postfix"
                              )
    tweaker = subparsers.add_parser("tweak", help="Tweak a builtin command")
    tweaker.add_argument('other',
                        choices=choices,
                        help="Command to tweak."
                        )
    commands.TweakAnotherCommand.config_argparse(tweaker)
    all_parser = subparsers.add_parser("ManualCommand", help="Customize a command")
    all_parser.add_argument('code', type=int,
                            help="The opcode to send to the pump."
                           )
    #__fields__ = ['maxRecords', 'code', 'descr',
    #            'serial', 'bytesPerRecord', 'retries', 'params']
    all_parser.add_argument('--params', type=str, action="append",
                            help="parameters to format into sent message",
                            default=commands.ManualCommand.params
                           )
    all_parser.add_argument('--descr', type=str, default="Experimental command",
                            help="Description of command"
                           )
    all_parser.add_argument('--name', type=str,
                            help="Proposed name of command"
                           )
    all_parser.add_argument('--save', action="store_true", default=False,
                            help="Save response in a file."
                           )
    all_parser.add_argument('--effectTime', type=float,
                            help="time to sleep before responding to message, float in seconds",
                            default=commands.ManualCommand.effectTime
                           )
    all_parser.add_argument('--maxRecords', type=int,
                            help="number of frames in a packet composing payload response",
                            default=commands.ManualCommand.maxRecords
                           )
    all_parser.add_argument('--bytesPerRecord', type=int,
                            help="bytes per frame",
                            default=commands.ManualCommand.bytesPerRecord
                           )

    return parser


if __name__ == '__main__':
  app = SendMsgApp( )
  app.run(None)




#!/usr/bin/env python
# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse
import sys, os
from decocare import link, stick, session, commands, lib
from pprint import pformat
import logging
log = logging.getLogger( ).getChild(__name__)

def parse_env ( ):
  return {
   "serial": os.environ.get('SERIAL', ''),
   "port": os.environ.get('PORT', '')
  }

def get_parser ( ):
  conf = parse_env( )
  parser = argparse.ArgumentParser( )
  parser.add_argument('--serial', type=str,
                      dest='serial',
                      default=conf.get('serial', ''),
                      help="serial number of pump [default: %(default)s]")
  parser.add_argument('--port', type=str,
                      dest='port',
                      default=conf.get('port', ''),
                      help="Path to device [default: %(default)s]")
  parser.add_argument('-v', '--verbose',
                      dest='verbose',
                      action='append_const', const=1,
                      help="Verbosity"
                      )
  parser.add_argument('--duration',
                      dest='duration',
                      type=int, default=0,
                      help="Duration of temp rate [default: %(default)s)]"
                      )
  parser.add_argument('--rate',
                      dest='rate',
                      type=float, default=0,
                      help="Rate of temp basal [default: %(default)s)]"
                      )
  argcomplete.autocomplete(parser)
  return parser

def format_params (args):
  duration = args.duration / 30
  rate = int(args.rate / 0.025)
  params = [0x00, rate, duration]
  return params

def main (args):
  print "## set a temporary basal rate"
  print "hi", "`", args, "`"
  print "```"
  uart = stick.Stick(link.Link(args.port, timeout=.400))
  print "```"
  print "```"
  uart.open( )
  print "```"
  print "```"
  pump = session.Pump(uart, args.serial)
  print "```"
  print "```"
  stats = uart.interface_stats( )
  print "```"
  print "```javascript"
  print pformat(stats)
  print "```"
  print "```"
  model = pump.read_model( )
  print "```"
  print '### PUMP MODEL: `%s`' % model

  print "### existing temp basal"
  print "```"
  reader = commands.ReadBasalTemp(serial=pump.serial)
  pump.execute(reader)
  temp_basal = reader.getData( )
  print "```"
  print "```javascript"
  print pformat(temp_basal)
  print "```"

  print "### setting rate"
  print "#### sending command"
  params = format_params(args)
  print "###### params: `%s`" % params
  print "```"
  #comm = commands.TempBasal(serial=device.serial, params=[ x ] )
  # params = [0x00, 0x01, 0x02]
  comm = commands.TempBasal(serial=pump.serial, params=params)
  pump.execute(comm)
  page = comm.getData( )
  print "```"
  print "### result"
  log.info("XXX: SET TempBasal!!:\n```\n%s\n```" % lib.hexdump(page))

  print "```"
  reader = commands.ReadBasalTemp(serial=pump.serial)
  pump.execute(reader)
  temp_basal = reader.getData( )
  print "```"
  print "###### results + params"
  print "```"
  print pformat(args)
  print lib.hexdump(params)
  print "```"
  print "```javascript"
  print pformat(temp_basal)
  print "```"
  print "```"
  stats = uart.interface_stats( )
  print "```"
  print "```javascript"
  print pformat(stats)
  print "```"

if __name__ == '__main__':
  parser = get_parser( )
  args = parser.parse_args( )
  level = None
  if args.verbose > 0:
    level = args.verbose > 1 and logging.DEBUG or logging.INFO
  logging.basicConfig(stream=sys.stdout, level=level)
  main(args)




#!/usr/bin/env python
# PYTHON_ARGCOMPLETE_OK
import argcomplete, argparse
import sys, os
from decocare import link, stick, session, commands, lib
from pprint import pformat
import logging
log = logging.getLogger( ).getChild(__name__)

def parse_env ( ):
  return {
   "serial": os.environ.get('SERIAL', ''),
   "port": os.environ.get('PORT', '')
  }

def get_parser ( ):
  conf = parse_env( )
  parser = argparse.ArgumentParser( )
  parser.add_argument('--serial', type=str,
                      dest='serial',
                      default=conf.get('serial', ''),
                      help="serial number of pump [default: %(default)s]")
  parser.add_argument('--port', type=str,
                      dest='port',
                      default=conf.get('port', ''),
                      help="Path to device [default: %(default)s]")
  parser.add_argument('--no-op',
                      dest='dryrun',
                      action='store_true', default=False,
                      help="Dry run, don't do main function"
                      )
  parser.add_argument('-v', '--verbose',
                      dest='verbose',
                      action='append_const', const=1,
                      help="Verbosity"
                      )
  parser.add_argument('--init',
                      dest='init',
                      action='store_true', default=False,
                      help="Send power ctrl to initialize RF session."
                      )
  parser.add_argument('commands',
                      nargs="+",
                      #dest='status',
                      choices=['act', 'esc', 'up', 'down', 'easy' ],
                      default='act',
                      help="buttons to press [default: %(default)s)]"
                      )
  argcomplete.autocomplete(parser)
  return parser

command_map = {
    'easy': commands.KeypadPush.EASY,
    'esc': commands.KeypadPush.ESC,
    'act': commands.KeypadPush.ACT,
    'down': commands.KeypadPush.DOWN,
    'up': commands.KeypadPush.UP,
}

def exec_request (pump, request):
  msg = command_map.get(request)
  response = pump.query(msg)
  print "response: %s" % response
  print "hexdump:"
  print "```"
  print lib.hexdump(response.data)
  print "```"
  print "##### decoded:\n```python\n", repr(response.getData( )), "\n```"

def main (args):
  print "## press keys on a pump"
  print "using", "`", args, "`"

  print "```"
  uart = stick.Stick(link.Link(args.port, timeout=.400))
  print "```"
  print "```"
  uart.open( )
  print "```"
  print "```"
  pump = session.Pump(uart, args.serial)
  print "```"
  print "```"
  stats = uart.interface_stats( )
  print "```"
  print "```javascript"
  print pformat(stats)
  print "```"
  print "```"
  if args.init:
    pump.power_control( )
  model = pump.read_model( )
  print "```"
  print '### PUMP MODEL: `%s`' % model

  for flow in args.commands:
    print '### ', flow
    if args.dryrun:
      print "#### dry run, no action taken"
    else:
      exec_request(pump, flow)

  print "### end stats"
  print "```"
  stats = uart.interface_stats( )
  print "```"
  print "```javascript"
  print pformat(stats)
  print "```"

if __name__ == '__main__':
  parser = get_parser( )
  args = parser.parse_args( )
  level = None
  if args.verbose > 0:
    level = args.verbose > 1 and logging.DEBUG or logging.INFO
  logging.basicConfig(stream=sys.stdout, level=level)
  main(args)




#!/usr/bin/python
"""
This module provides some basic helper/formatting utilities,
specifically targeted at decoding ReadHistoryData data.

"""
import sys
from binascii import hexlify

import lib
from records import *

_remote_ids = [
  bytearray([ 0x01, 0xe2, 0x40 ]),
  bytearray([ 0x03, 0x42, 0x2a ]),
  bytearray([ 0x0c, 0x89, 0x92 ]),
]

def decode_remote_id(msg):
  """
  practice decoding some remote ids:

  | 0x27
  | 0x01 0xe2 0x40
  | 0x03 0x42 0x2a
  | 0x28 0x0c 0x89
  | 0x92 0x00 0x00 0x00

  >>> decode_remote_id(_remote_ids[0])
  '123456'

  >>> decode_remote_id(_remote_ids[1])
  '213546'

  >>> decode_remote_id(_remote_ids[2])
  '821650'



  """
  high   = msg[ 0 ] * 256 * 256
  middle = msg[ 1 ] * 256
  low    = msg[ 2 ]
  return str(high + middle + low)

class NoDelivery(KnownRecord):
  opcode = 0x06
  head_length = 4
#class ResultTotals(KnownRecord):
class MResultTotals(InvalidRecord):
  """On 722 this seems like two records."""
  opcode = 0x07
  #head_length = 5
  head_length = 5
  date_length = 2
  #body_length = 37 + 4
  #body_length = 2
  def __init__(self, head, larger=False):
    super(type(self), self).__init__(head, larger)
    if larger:
      self.body_length = 3
  def parse_time(self):
    mid = unmask_m_midnight(self.date)
    try:
      self.datetime = date = datetime(*mid)
      return date
    except ValueError, e:
      print "ERROR", e, mid, lib.hexdump(self.date)
      pass
    return mid
      
    
  def date_str(self):
    result = 'unknown'
    if self.datetime is not None:
      result = self.datetime.isoformat( )
    else:
      if len(self.date) >=2:
        result = "{}".format(unmask_m_midnight(self.date))
    return result

class ChangeBasalProfile(KnownRecord):
  opcode = 0x08
  body_length = 44
class ClearAlarm(KnownRecord):
  opcode = 0x0C
class SelectBasalProfile(KnownRecord):
  opcode = 0x14
class ChangeTime(KnownRecord):
  opcode = 0x17
class NewTimeSet(KnownRecord):
  opcode = 0x18
class LowBattery(KnownRecord):
  opcode = 0x19
class Battery(KnownRecord):
  opcode = 0x1a
class PumpSuspend(KnownRecord):
  opcode = 0x1e
class PumpResume(KnownRecord):
  opcode = 0x1f

class Rewind(KnownRecord):
  opcode = 0x21
class EnableDisableRemote(KnownRecord):
  opcode = 0x26
  body_length = 14
class ChangeRemoteID(KnownRecord):
  opcode = 0x27

class TempBasalDuration(KnownRecord):
  opcode = 0x16
  _test_1 = bytearray([ ])
  def decode(self):
    self.parse_time( )
    basal = { 'duration (min)': self.head[1] * 30, }
    return basal
class TempBasal(KnownRecord):
  opcode = 0x33
  body_length = 1
  _test_1 = bytearray([ ])
  def decode(self):
    self.parse_time( )
    basal = { 'rate': self.head[1] / 40.0, }
    return basal

class LowReservoir(KnownRecord):
  """
  >>> rec = LowReservoir( LowReservoir._test_1[:2] )
  >>> decoded = rec.parse(LowReservoir._test_1)
  >>> print str(rec)
  LowReservoir 2012-12-07T11:02:43 head[2], body[0] op[0x34]

  >>> print pformat(decoded)
  {'amount': 20.0}
  """
  opcode = 0x34
  _test_1 = bytearray([ 0x34, 0xc8,
                        0xeb, 0x02, 0x0b, 0x07, 0x0c, ])
  def decode(self):
    self.parse_time( )
    reservoir = {'amount' : int(self.head[1]) / 10.0 }
    return reservoir


class ChangeUtility(KnownRecord):
  opcode = 0x63
class ChangeTimeDisplay(KnownRecord):
  opcode = 0x64

_confirmed = [ Bolus, Prime, NoDelivery, MResultTotals, ChangeBasalProfile,
               ClearAlarm, SelectBasalProfile, TempBasalDuration, ChangeTime,
               NewTimeSet, LowBattery, Battery, PumpSuspend,
               PumpResume, CalBGForPH, Rewind, EnableDisableRemote,
               ChangeRemoteID, TempBasal, LowReservoir, BolusWizard,
               UnabsorbedInsulinBolus, ChangeUtility, ChangeTimeDisplay ]

class Ian69(KnownRecord):
  opcode = 0x69
  body_length = 2
_confirmed.append(Ian69)

class Ian50(KnownRecord):
  opcode = 0x50
  body_length = 34
_confirmed.append(Ian50)

class Ian54(KnownRecord):
  opcode = 0x54
  body_length = 34 + 23
  body_length = 57
_confirmed.append(Ian54)

class Ian0B(KnownRecord):
  opcode = 0x0B
  head_length = 3
_confirmed.append(Ian0B)

class Ian3F(KnownRecord):
  opcode = 0x3F
  body_length = 3
_confirmed.append(Ian3F)

class IanA8(KnownRecord):
  opcode = 0xA8
  head_length = 10
_confirmed.append(IanA8)

class BasalProfileStart(KnownRecord):
  opcode = 0x7b
  body_length = 3
_confirmed.append(BasalProfileStart)

class old6c(InvalidRecord):
  opcode = 0x6c
  #head_length = 45
  body_length = 38
  # body_length = 34
_confirmed.append(old6c)

class Model522ResultTotals(KnownRecord):
  opcode = 0x6d
  head_length = 1
  date_length = 2
  body_length = 40
  def parse_time(self):
    mid = unmask_m_midnight(self.date)
    try:
      self.datetime = date = datetime(*mid)
      return date
    except ValueError, e:
      print "ERROR", e, lib.hexdump(self.date)
      pass
    return mid
      
    
  def date_str(self):
    result = 'unknown'
    if self.datetime is not None:
      result = self.datetime.isoformat( )
    else:
      if len(self.date) >=2:
        result = "{}".format(unmask_m_midnight(self.date))
    return result

def unmask_m_midnight(data):
  """
  Extract date values from a series of bytes.
  Always returns tuple given a bytearray of at least 3 bytes.

  Returns 6-tuple of scalar values year, month, day, hours, minutes,
  seconds.

  """
  data = data[:]
  seconds = 0
  minutes = 0
  hours   = 0

  day     = parse_day(data[0])

  high = data[0] >> 4
  low  = data[0] & 0x1F

  year_high = data[1] >> 4
  # month = int(high) #+ year_high
  # month   = parse_months( data[0], data[1] )
  mhigh = (data[0] & 0xE0) >> 4
  mlow  = (data[1] & 0x80) >> 7
  month =  int(mhigh + mlow)
  day = int(low) + 1

  year = parse_years(data[1])
  return (year, month, day, hours, minutes, seconds)

_confirmed.append(Model522ResultTotals)

class Sara6E(Model522ResultTotals):
  """Seems specific to 722?"""
  opcode = 0x6e
  #head_length = 52 - 5
  # body_length = 1
  body_length = 48
  #body_length = 0
  def __init__(self, head, larger=False):
    super(type(self), self).__init__(head, larger)
    if larger:
      self.body_length = 48
_confirmed.append(Sara6E)

_known = { }

_variant = { }

for x in _confirmed:
  _known[x.opcode] = x

del x

def suggest(head, larger=False):
  """
  Look in the known table of commands to find a suitable record type
  for this opcode.
  """
  klass = _known.get(head[0], Base)
  record = klass(head, larger)
  return record

def parse_record(fd, head=bytearray( ), larger=False):
  """
  Given a file-like object, and the head of a record, parse the rest
  of the record.
  Look up the type of record, read in just enough data to parse it,
  return the result.
  """
  # head    = bytearray(fd.read(2))
  date    = bytearray( )
  body    = bytearray( )
  record  = suggest(head, larger)
  remaining = record.head_length - len(head)
  if remaining > 0:
    head.extend(bytearray(fd.read(remaining)))
  if record.date_length > 0:
    date.extend(bytearray(fd.read(record.date_length)))
  if record.body_length > 0:
    body.extend(bytearray(fd.read(record.body_length)))
  record.parse( head + date + body )
  # print str(record)
  # print record.pformat(prefix=str(record) )
  return record


def describe( ):
  keys = _known.keys( )
  out  = [ ]
  for k in keys:
    out.append(_known[k].describe( ))
  return out

    

if __name__ == '__main__':
  import doctest
  doctest.testmod( )

#####
# EOF
